From c804d69b6daad66d9d732b0f8adaf226af3641c8 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Marc=20Nieper-Wi=C3=9Fkirchen?= <marc@nieper-wisskirchen.de>
Date: Sat, 31 Dec 2022 14:08:53 +0100
Subject: [PATCH] Display evaluation errors for Scheme code blocks.

---
 lisp/ob-eval.el   |  9 ++++++---
 lisp/ob-scheme.el | 21 +++++++++++++++------
 2 files changed, 21 insertions(+), 9 deletions(-)

diff --git a/lisp/ob-eval.el b/lisp/ob-eval.el
index f57566079..4f31b4109 100644
--- a/lisp/ob-eval.el
+++ b/lisp/ob-eval.el
@@ -37,16 +37,19 @@
 (declare-function org-babel-temp-file "ob-core" (prefix &optional suffix))
 
 (defun org-babel-eval-error-notify (exit-code stderr)
-  "Open a buffer to display STDERR and a message with the value of EXIT-CODE."
+  "Open a buffer to display STDERR and a message with the value of EXIT-CODE.
+If EXIT-CODE is nil, suppress the message."
   (let ((buf (get-buffer-create org-babel-error-buffer-name)))
     (with-current-buffer buf
       (goto-char (point-max))
       (save-excursion
         (unless (bolp) (insert "\n"))
         (insert stderr)
-        (insert (format "[ Babel evaluation exited with code %S ]" exit-code))))
+        (when exit-code
+          (insert (format "[ Babel evaluation exited with code %S ]" exit-code)))))
     (display-buffer buf))
-  (message "Babel evaluation exited with code %S" exit-code))
+  (when exit-code
+    (message "Babel evaluation exited with code %S" exit-code)))
 
 (defun org-babel-eval (command query)
   "Run COMMAND on QUERY.
diff --git a/lisp/ob-scheme.el b/lisp/ob-scheme.el
index 5be13bd89..e7c9db48d 100644
--- a/lisp/ob-scheme.el
+++ b/lisp/ob-scheme.el
@@ -4,7 +4,6 @@
 
 ;; Authors: Eric Schulte
 ;;        Michael Gauland
-;; Maintainer: Rudolf Adamkovič <salutis@me.com>
 ;; Keywords: literate programming, reproducible research, scheme
 ;; URL: https://orgmode.org
 
@@ -66,6 +65,8 @@
 (declare-function geiser-repl-exit "ext:geiser-repl" (&optional arg))
 (declare-function geiser-eval--retort-output "ext:geiser-eval" (ret))
 (declare-function geiser-eval--retort-result-str "ext:geiser-eval" (ret prefix))
+(declare-function geiser-eval--retort-error "ext:geiser-eval" (ret))
+(declare-function geiser-eval--retort-error-msg "ext:geiser-eval" (err))
 
 (defcustom org-babel-scheme-null-to 'hline
   "Replace `null' and empty lists in scheme tables with this before returning."
@@ -194,22 +195,30 @@ is true; otherwise returns the last value."
                                   #'geiser-eval-region)
                                 (point-min)
                                 (point-max))))
-	      (setq result (if output
+              (let ((err (geiser-eval--retort-error ret)))
+                (setq result (cond
+                              (err nil)
+                              (output
                                (or (geiser-eval--retort-output ret)
-				   "Geiser Interpreter produced no output")
-			     (geiser-eval--retort-result-str ret "")))))
+                                   "Geiser Interpreter produced no output"))
+                              (t (geiser-eval--retort-result-str ret ""))))
                 (when (not repl)
                   (save-current-buffer (set-buffer repl-buffer)
                                        (geiser-repl-exit))
                   (set-process-query-on-exit-flag (get-buffer-process repl-buffer) nil)
-	    (kill-buffer repl-buffer)))))
+                  (kill-buffer repl-buffer))
+                (when err
+                  (let ((msg (geiser-eval--error-msg err)))
+                    (org-babel-eval-error-notify
+                     nil
+                     (concat (if (listp msg) (car msg) msg) "\n"))))))))))
     result))
 
 (defun org-babel-scheme--table-or-string (results)
   "Convert RESULTS into an appropriate elisp value.
 If the results look like a list or tuple, then convert them into an
 Emacs-lisp table, otherwise return the results as a string."
-  (let ((res (org-babel-script-escape results)))
+  (let ((res (and results (org-babel-script-escape results))))
     (cond ((listp res)
            (mapcar (lambda (el)
                      (if (or (null el) (eq el 'null))
-- 
2.34.1

